import os
import shutil
import tarfile
import pandas as pd
from chazutsu.datasets.framework.dataset import Dataset
from chazutsu.datasets.framework.resource import Resource


class IMDB(Dataset):

    def __init__(self):
        super().__init__(
            name="Large Movie Review Dataset(IMDB)",
            site_url="http://ai.stanford.edu/~amaas/data/sentiment/",
            download_url="http://ai.stanford.edu/~amaas/data/sentiment/aclImdb_v1.tar.gz",
            description="Movie review data that is constructed by 25,000 train/test reviews that have positive/negative annotation"
            )
    
    def download(self, directory="", shuffle=True, test_size=0, sample_count=0, keep_raw=False):
        if test_size != 0:
            raise Exception("This dataset is already splitted to train & test.")
        
        return super().download(directory, shuffle, 0, sample_count, keep_raw)
    
    def extract(self, path):
        dir, file_name = os.path.split(path)
        work_dir = os.path.join(dir, "tmp")
        if not os.path.isdir(work_dir):
            with tarfile.open(path) as t:
                t.extractall(path=work_dir)
        
        extracted_dir = os.path.join(work_dir, "aclImdb")
        data_dirs = ["train", "test"]
        pathes = []
        for d in data_dirs:
            target_dir = os.path.join(extracted_dir, d)
            file_path = os.path.join(dir, "imdb_" + d + ".txt")
            self.label_by_dir(file_path, target_dir, {"pos": 1, "neg": 0}, task_size=1000)

            pathes.append(file_path)

            if d == "train":
                unlabeled = os.path.join(dir, "imdb_unlabeled.txt")
                self.label_by_dir(unlabeled, target_dir, {"unsup": None}, task_size=1000)
                pathes.append(unlabeled)

        os.remove(path)
        shutil.rmtree(work_dir)

        return pathes[0]

    def make_resource(self, data_root):
        return IMDBResource(data_root)

    @classmethod
    def _parallel_parser(cls, label, path):
        features = cls._file_to_features(path)
        if label is not None:
            line = "\t".join([str(label)] + features) + "\n"
        else:
            line = "\t".join(features) + "\n"  # unlabeled
        return line

    @classmethod
    def _file_to_features(cls, path):
        # override this method if you want implements custome process
        fs = []
        file_name = os.path.basename(path)
        f, ext = os.path.splitext(file_name)
        els = f.split("_")
        rating = 0
        if len(els) == 2:
            rating = els[-1]
        
        review = ""
        with open(path, encoding="utf-8") as f:
            lines = f.readlines()
            lines = [ln.replace("\t", " ").strip() for ln in lines]
            review = " ".join(lines)
        
        if rating != "0":
            return [rating, review]
        else:
            return [review]


class IMDBResource(Resource):

    def __init__(self, 
        root,
        train_file_suffix="_train",
        test_file_suffix="_test",
        sample_file_suffix="_samples"):

        super().__init__(
            root, 
            ["polarity", "rating", "review"],
            "polarity",
            train_file_suffix, 
            test_file_suffix, 
            sample_file_suffix)
        
        self.path = self.train_file_path
        self.unlabeled_data_path = ""
        for f in os.listdir(self.root):
            p = os.path.join(self.root, f)
            n, e = os.path.splitext(f)
            if n.endswith("_unlabeled"):
                self.unlabeled_data_path = p
                break
    
    def unlabeled_data(self):
        df = pd.read_table(self.unlabeled_data_path, header=None, names=["review"])
        return df



import os
import re
import tarfile
import shutil
from tqdm import tqdm
from chazutsu.datasets.framework.dataset import Dataset
from chazutsu.datasets.framework.resource import Resource


class NewsGroup20(Dataset):

    def __init__(self, group_filter=()):
        super().__init__(
            name="20 Newsgroups",
            site_url="http://qwone.com/~jason/20Newsgroups/",
            download_url="http://qwone.com/~jason/20Newsgroups/20news-18828.tar.gz",
            description="news article and its comments that are categorized by 20 group."
            )
        
        self.group_filter = group_filter
        self._mail_pattern = re.compile("[\w|\.]+@[\w|\.]+")

    def extract(self, path):
        dir, file_name = os.path.split(path)
        work_dir = os.path.join(dir, "tmp")
        newsgroup20_path = os.path.join(dir, "newsgroup20.txt")

        if not os.path.isdir(work_dir):
            with tarfile.open(path) as t:
                t.extractall(path=work_dir)

        dataset_path = os.path.join(work_dir, "20news-18828")
        with open(newsgroup20_path, mode="wb") as f:
            for gp in os.listdir(dataset_path):
                group_path = os.path.join(dataset_path, gp)
                if not os.path.isdir(group_path):
                    continue
                if len(self.group_filter) > 0 and gp not in self.group_filter:
                    continue

                self.logger.info("Extracting {} news data.".format(gp))
                for news in tqdm(os.listdir(group_path)):
                    group_name = gp
                    category_name = self.get_category(gp)
                    subject, author, text = self.parse(path=os.path.join(group_path, news))
                    ln = "\t".join([
                        group_name,
                        category_name,
                        subject,
                        author,
                        text
                    ]) + "\n"
                    f.write(ln.encode("utf-8"))

        # remove files
        os.remove(path)
        shutil.rmtree(work_dir)

        return newsgroup20_path

    def make_resource(self, data_root):
        return Resource(data_root, columns=["group", "group-category", "subject", "author", "text"], target="group")

    def get_category(self, group_name):
        g = group_name
        if g.startswith("talk."):
            g = g.replace("talk.", "")

        cats = group_name.split(".")
        category_name = cats[0]
        if g in ["alt.atheism", "soc.religion.christian"]:
            category_name = "religion"
        
        return category_name
    
    def parse(self, path="", raw_text=""):
        body = raw_text

        if body:
            body = body.split("\n")
        elif path:
            with open(path, errors="ignore", encoding="utf-8") as f:
                body = f.readlines()
        else:
            raise Exception("Can not get parse target text.")
        
        def strip(s):
            _s = s
            for c in ["<", ">", "^", "-", "(", ")", "*"]:
                _s = _s.replace(c , "")

            _s = re.sub(self._mail_pattern, "", _s)
            _s = _s.replace("\t", " ")
            _s = _s.strip()
            if not _s:
                return ""
            elif _s.endswith("writes:"):
                return ""
            elif _s.startswith("Archive-name:"):
                return ""
            elif _s.startswith("Alt-atheism-archive-name:"):
                return ""
            elif _s.startswith("Last-modified:"):
                return ""
            elif _s.startswith("Version:"):
                return ""
            else:
                return _s

        body = [s for s in [strip(s) for s in body] if s]
        subject = ""
        author = ""
        text = ""

        for i, s in enumerate(body):
            els = s.split(":")
            if len(els) < 2:
                continue

            if els[0].startswith("From"):
                author = ":".join(els[1:]).strip()
            elif els[0].startswith("Subject"):
                subject = ":".join(els[1:]).strip()
            
            if author and subject:
                break

            if i > 2:
                break  # can not find out
        
        text = " ".join(body[2:])

        return subject, author, text



from setuptools import setup



setup(
    name = "chazutsu",
    version = "0.3.2",
    description = "The tool to make NLP datasets ready to use",
    keywords = ["machine learning", "nlp", "natural language processing"],
    author = "icoxfog417",
    author_email = "icoxfog417@yahoo.co.jp",
    license="Apache License 2.0",
    packages = [
        "chazutsu",
        "chazutsu.datasets",
        "chazutsu.datasets.framework"
        ],
    url = "https://github.com/chakki-works/chazutsu",
    install_requires=[
        "requests>=2.14.2",
        "tqdm>=4.11.2",
        "pandas>=0.20.1",
        "joblib>=0.11",
    ],
)


import os
import pandas as pd


class Resource():

    def __init__(self, 
        root,
        columns=None,
        target="",
        train_file_suffix="_train",
        test_file_suffix="_test",
        sample_file_suffix="_samples"):

        self.root = root
        self.columns = columns
        self.target = target
        self.path = ""
        self.train_file_path = ""
        self.test_file_path = ""
        self.sample_file_path = ""

        if target and target not in columns:
            raise Exception("Target have to be selected from columns.")

        self._file_pathes = []
        for f in os.listdir(self.root):
            p = os.path.join(self.root, f)
            if not os.path.isfile(p):
                continue
            
            n, e = os.path.splitext(f)
            if n.endswith(train_file_suffix):
                self.train_file_path = p
            elif n.endswith(test_file_suffix):
                self.test_file_path = p
            elif n.endswith(sample_file_suffix):
                self.sample_file_path = p
            elif not self.path:
                self.path = p
        
        if not self.path:
            self.path = self.train_file_path

    def data(self, split_target=False):
        if self.path:
            return self._to_pandas(self.path, split_target)
        elif self.train_file_path:
            return self._to_pandas(self.train_file_path, split_target)

    def train_data(self, split_target=False):
        return self._to_pandas(self.train_file_path, split_target)

    def test_data(self, split_target=False):
        return self._to_pandas(self.test_file_path, split_target)

    def sample_data(self, split_target=False):
        return self._to_pandas(self.sample_file_path, split_target)

    def _to_pandas(self, path, split_target):
        df = pd.read_table(path, header=None, names=self.columns)

        if not split_target:
            return df
        else:
            target = df[self.target]
            df.drop(self.target, axis=1, inplace=True)
            return target, df
